@@ -1,7 +1,6 @@
 use nalgebra::{storage::StorageMut, Vector};
 
-use super::domain::Domain;
-use super::system::System;
+use super::{domain::Domain, system::System};
 
 /// Common interface for all solvers.
 ///
@@ -22,7 +21,7 @@ use super::system::System;
 ///
 /// ```rust
 /// use gomez::nalgebra as na;
-/// use gomez::core::{Domain, Solver, System, SystemError};
+/// use gomez::core::*;
 /// use na::{storage::StorageMut, Vector};
 /// use rand::Rng;
 /// use rand_distr::{uniform::SampleUniform, Distribution, Uniform};
@@ -42,7 +41,7 @@ use super::system::System;
 ///     F::Scalar: SampleUniform,
 /// {
 ///     const NAME: &'static str = "Random";
-///     type Error = SystemError;
+///     type Error = Error;
 ///
 ///     fn next<Sx, Sfx>(
 ///         &mut self,
@@ -61,7 +60,7 @@ use super::system::System;
 ///         });
 ///
 ///         // We must compute the residuals.
-///         f.apply_mut(x, fx)?;
+///         f.eval(x, fx)?;
 ///
 ///         Ok(())
 ///     }
@@ -73,7 +72,7 @@ pub trait Solver<F: System> {
 
     /// Error type of the iteration. Represents an invalid operation during
     /// computing the next step. It is usual that one of the error kinds is
-    /// propagation of the [`SystemError`](super::SystemError) from the system.
+    /// propagation of the [`Error`](super::Error) from the system.
     type Error;
 
     /// Computes the next step in the solving process.
@@ -81,7 +80,7 @@ pub trait Solver<F: System> {
     /// The value of `x` is the current values of variables. After the method
     /// returns, `x` should hold the variable values of the performed step and
     /// `fx` *must* contain residuals of that step as computed by
-    /// [`System::apply_mut`].
+    /// [`System::eval`].
     ///
     /// It is implementation error not to compute the residuals of the computed
     /// step.
