@@ -29,7 +29,10 @@ use num_traits::{One, Zero};
 use thiserror::Error;
 
 use crate::{
-    core::{Domain, Solver, System, SystemError, SystemExt, VectorDomainExt},
+    core::{
+        Domain, Error, Function, FunctionResultExt, Optimizer, Problem, Solver, System,
+        VectorDomainExt,
+    },
     derivatives::EPSILON_SQRT,
 };
 
@@ -50,7 +53,7 @@ pub enum CoefficientsFamily {
 /// Options for [`NelderMead`] solver.
 #[derive(Debug, Clone, CopyGetters, Setters)]
 #[getset(get_copy = "pub", set = "pub")]
-pub struct NelderMeadOptions<F: System> {
+pub struct NelderMeadOptions<F: Problem> {
     /// Family for coefficients adaptation or fixed coefficients. Default:
     /// balanced (see [`CoefficientsFamily`]).
     family: CoefficientsFamily,
@@ -66,7 +69,7 @@ pub struct NelderMeadOptions<F: System> {
     shrink_coeff: F::Scalar,
 }
 
-impl<F: System> Default for NelderMeadOptions<F> {
+impl<F: Problem> Default for NelderMeadOptions<F> {
     fn default() -> Self {
         Self {
             family: CoefficientsFamily::Balanced,
@@ -79,7 +82,7 @@ impl<F: System> Default for NelderMeadOptions<F> {
     }
 }
 
-impl<F: System> NelderMeadOptions<F> {
+impl<F: Problem> NelderMeadOptions<F> {
     fn overwrite_coeffs(&mut self, f: &F) {
         let Self {
             family,
@@ -116,12 +119,14 @@ impl<F: System> NelderMeadOptions<F> {
 }
 
 /// Nelder-Mead solver. See [module](self) documentation for more details.
-pub struct NelderMead<F: System>
+pub struct NelderMead<F: Problem>
 where
     DefaultAllocator: Allocator<F::Scalar, F::Dim>,
     DefaultAllocator: Allocator<F::Scalar, F::Dim, F::Dim>,
 {
     options: NelderMeadOptions<F>,
+    fx: OVector<F::Scalar, F::Dim>,
+    fx_best: OVector<F::Scalar, F::Dim>,
     scale: OVector<F::Scalar, F::Dim>,
     centroid: OVector<F::Scalar, F::Dim>,
     reflection: OVector<F::Scalar, F::Dim>,
@@ -132,7 +137,7 @@ where
     sort_perm: Vec<usize>,
 }
 
-impl<F: System> NelderMead<F>
+impl<F: Problem> NelderMead<F>
 where
     DefaultAllocator: Allocator<F::Scalar, F::Dim>,
     DefaultAllocator: Allocator<F::Scalar, F::Dim, F::Dim>,
@@ -150,6 +155,8 @@ where
 
         Self {
             options,
+            fx: OVector::zeros_generic(f.dim(), U1::name()),
+            fx_best: OVector::zeros_generic(f.dim(), U1::name()),
             scale: OVector::from_iterator_generic(f.dim(), U1::name(), scale_iter),
             centroid: OVector::zeros_generic(f.dim(), U1::name()),
             reflection: OVector::zeros_generic(f.dim(), U1::name()),
@@ -160,6 +167,14 @@ where
             sort_perm: Vec::with_capacity(f.dim().value() + 1),
         }
     }
+
+    /// Resets the internal state of the solver.
+    pub fn reset(&mut self) {
+        // Causes simplex to be initialized again.
+        self.simplex.clear();
+        self.errors.clear();
+        self.sort_perm.clear();
+    }
 }
 
 /// Error returned from [`NelderMead`] solver.
@@ -167,31 +182,25 @@ where
 pub enum NelderMeadError {
     /// Error that occurred when evaluating the system.
     #[error("{0}")]
-    System(#[from] SystemError),
+    Problem(#[from] Error),
     /// Simplex collapsed so it is impossible to make any progress.
     #[error("simplex collapsed")]
     SimplexCollapsed,
 }
 
-impl<F: System> Solver<F> for NelderMead<F>
+impl<F: Function> NelderMead<F>
 where
     DefaultAllocator: Allocator<F::Scalar, F::Dim>,
     DefaultAllocator: Allocator<F::Scalar, F::Dim, F::Dim>,
 {
-    const NAME: &'static str = "Nelder-Mead";
-
-    type Error = NelderMeadError;
-
-    fn next<Sx, Sfx>(
+    fn next_inner<Sx>(
         &mut self,
         f: &F,
-        dom: &Domain<F::Scalar>,
-        x: &mut Vector<F::Scalar, F::Dim, Sx>,
-        fx: &mut Vector<F::Scalar, F::Dim, Sfx>,
-    ) -> Result<(), Self::Error>
+        dom: &Domain<<F>::Scalar>,
+        x: &mut Vector<<F>::Scalar, <F>::Dim, Sx>,
+    ) -> Result<F::Scalar, NelderMeadError>
     where
-        Sx: StorageMut<F::Scalar, F::Dim>,
-        Sfx: StorageMut<F::Scalar, F::Dim>,
+        Sx: StorageMut<<F>::Scalar, <F>::Dim>,
     {
         let NelderMeadOptions {
             reflection_coeff,
@@ -203,6 +212,8 @@ where
         } = self.options;
 
         let Self {
+            fx,
+            fx_best,
             scale,
             simplex,
             errors,
@@ -215,22 +226,61 @@ where
         } = self;
 
         let n = f.dim().value();
+        let inf = convert(f64::INFINITY);
 
         if simplex.is_empty() {
             // Simplex initialization.
+
+            // It is important to return early on error before the point is
+            // added to the simplex.
+            let mut error_best = f.apply(x)?;
+            errors.push(error_best);
             simplex.push(x.clone_owned());
-            errors.push(f.apply_mut_norm_squared(x, fx)?);
 
             for j in 0..n {
                 let mut xi = x.clone_owned();
                 xi[j] = dom.vars()[j].clamp(xi[j] + scale[j]);
 
-                errors.push(f.apply_mut_norm_squared(&xi, fx)?);
+                let error = match f.apply(&xi) {
+                    Ok(error) => error,
+                    // Do not fail when invalid value is encountered during
+                    // building the simplex. Instead, treat it as infinity so
+                    // that it is worse than (or "equal" to) any other error and
+                    // hope that it gets replaced. The exception is the very
+                    // point provided by the caller as it is expected to be
+                    // valid and it is erroneous situation if it is not.
+                    Err(Error::InvalidValue) => inf,
+                    Err(error) => {
+                        // Clear the simplex so the solver is not in invalid
+                        // state.
+                        simplex.clear();
+                        errors.clear();
+                        return Err(error.into());
+                    }
+                };
+
+                if error < error_best {
+                    fx_best.copy_from(fx);
+                    error_best = error;
+                }
+
+                errors.push(error);
                 simplex.push(xi);
             }
 
+            let inf_error_count = errors.iter().filter(|e| **e == inf).count();
+
+            if inf_error_count >= n / 2 {
+                // The simplex is too degenerate.
+                simplex.clear();
+                errors.clear();
+                return Err(NelderMeadError::Problem(Error::InvalidValue));
+            }
+
             sort_perm.extend(0..=n);
-            sort_perm.sort_unstable_by(|a, b| {
+            // Stable sort is important for sort_perm[0] being consistent with
+            // fx_best.
+            sort_perm.sort_by(|a, b| {
                 errors[*a]
                     .partial_cmp(&errors[*b])
                     .unwrap_or(std::cmp::Ordering::Equal)
@@ -268,26 +318,30 @@ where
         // Perform one of possible simplex transformations.
         reflection.on_line2_mut(centroid, &simplex[sort_perm[n]], reflection_coeff);
         let reflection_not_feasible = reflection.project(dom);
-        let reflection_error = f.apply_mut_norm_squared(reflection, fx)?;
+        let reflection_error = f.apply_eval(reflection, fx).ignore_invalid_value(inf)?;
 
         #[allow(clippy::suspicious_else_formatting)]
         let (transformation, not_feasible) = if errors[sort_perm[0]] <= reflection_error
             && reflection_error < errors[sort_perm[n - 1]]
         {
-            // Reflected point is neither best nor worst in te new simplex. Just
-            // replace the worst point.
+            // Reflected point is neither best nor worst in the new simplex.
+            // Just replace the worst point.
             simplex[sort_perm[n]].copy_from(reflection);
             errors[sort_perm[n]] = reflection_error;
             (Transformation::Reflection, reflection_not_feasible)
         } else if reflection_error < errors[sort_perm[0]] {
+            fx_best.copy_from(fx);
+
             // Reflected point is better than the current best. Try to go
             // farther along this direction.
             expansion.on_line2_mut(centroid, &simplex[sort_perm[n]], expansion_coeff);
             let expansion_not_feasible = expansion.project(dom);
-            let expansion_error = f.apply_mut_norm_squared(expansion, fx)?;
+            let expansion_error = f.apply_eval(expansion, fx).ignore_invalid_value(inf)?;
 
             if expansion_error < reflection_error {
-                // Expansion indeed help, replace the worst point.
+                fx_best.copy_from(fx);
+
+                // Expansion indeed helped, replace the worst point.
                 simplex[sort_perm[n]].copy_from(expansion);
                 errors[sort_perm[n]] = expansion_error;
                 (Transformation::Expansion, expansion_not_feasible)
@@ -309,9 +363,13 @@ where
                 // Try to perform outer contraction.
                 contraction.on_line2_mut(centroid, &simplex[sort_perm[n]], outer_contraction_coeff);
                 let contraction_not_feasible = contraction.project(dom);
-                let contraction_error = f.apply_mut_norm_squared(contraction, fx)?;
+                let contraction_error = f.apply_eval(contraction, fx).ignore_invalid_value(inf)?;
 
                 if contraction_error <= reflection_error {
+                    if contraction_error < errors[sort_perm[0]] {
+                        fx_best.copy_from(fx);
+                    }
+
                     // Use the contracted point instead of the reflected point
                     // because it's better.
                     simplex[sort_perm[n]].copy_from(contraction);
@@ -327,9 +385,13 @@ where
                 // Try to perform inner contraction.
                 contraction.on_line2_mut(centroid, &simplex[sort_perm[n]], inner_contraction_coeff);
                 let contraction_not_feasible = contraction.project(dom);
-                let contraction_error = f.apply_mut_norm_squared(contraction, fx)?;
+                let contraction_error = f.apply_eval(contraction, fx).ignore_invalid_value(inf)?;
 
                 if contraction_error <= errors[sort_perm[n]] {
+                    if contraction_error < errors[sort_perm[0]] {
+                        fx_best.copy_from(fx);
+                    }
+
                     // The contracted point is better than the worst point.
                     simplex[sort_perm[n]].copy_from(contraction);
                     errors[sort_perm[n]] = contraction_error;
@@ -349,10 +411,18 @@ where
                     // Shrink the simplex towards the best point.
 
                     contraction.copy_from(&simplex[sort_perm[0]]);
+                    let mut error_best = errors[sort_perm[0]];
+
                     for i in 1..=n {
                         let xi = &mut simplex[sort_perm[i]];
                         xi.on_line_mut(contraction, shrink_coeff);
-                        errors[sort_perm[i]] = f.apply_mut_norm_squared(xi, fx)?;
+                        let error = f.apply_eval(xi, fx).ignore_invalid_value(inf)?;
+                        errors[sort_perm[i]] = error;
+
+                        if error < error_best {
+                            fx_best.copy_from(fx);
+                            error_best = error;
+                        }
                     }
 
                     (Transformation::Shrinkage, false)
@@ -360,15 +430,16 @@ where
             }
         };
 
-        // Establish the ordering of simplex points.
-        sort_perm.sort_unstable_by(|a, b| {
+        // Establish the ordering of simplex points. Stable sort is important
+        // for sort_perm[0] being consistent with fx_best.
+        sort_perm.sort_by(|a, b| {
             errors[*a]
                 .partial_cmp(&errors[*b])
                 .unwrap_or(std::cmp::Ordering::Equal)
         });
 
         debug!(
-            "performed {}{}, error range: {} - {}",
+            "performed {}{},\t|| fx || = {} - {}",
             transformation.as_str(),
             if not_feasible { " with projection" } else { "" },
             errors[sort_perm[0]],
@@ -377,7 +448,7 @@ where
 
         // Return the best simplex point.
         x.copy_from(&simplex[sort_perm[0]]);
-        f.apply_mut(x, fx)?;
+        // fx corresponding to x is stored in `self.fx_best`.
 
         if transformation == Transformation::Shrinkage || not_feasible {
             // Check whether the simplex collapsed or not. It can happen only
@@ -397,7 +468,55 @@ where
             }
         }
 
-        Ok(())
+        Ok(errors[sort_perm[0]])
+    }
+}
+
+impl<F: Function> Optimizer<F> for NelderMead<F>
+where
+    DefaultAllocator: Allocator<F::Scalar, F::Dim>,
+    DefaultAllocator: Allocator<F::Scalar, F::Dim, F::Dim>,
+{
+    const NAME: &'static str = "Nelder-Mead";
+
+    type Error = NelderMeadError;
+
+    fn next<Sx>(
+        &mut self,
+        f: &F,
+        dom: &Domain<F::Scalar>,
+        x: &mut Vector<F::Scalar, F::Dim, Sx>,
+    ) -> Result<F::Scalar, Self::Error>
+    where
+        Sx: StorageMut<F::Scalar, F::Dim>,
+    {
+        self.next_inner(f, dom, x)
+    }
+}
+
+impl<F: System + Function> Solver<F> for NelderMead<F>
+where
+    DefaultAllocator: Allocator<F::Scalar, F::Dim>,
+    DefaultAllocator: Allocator<F::Scalar, F::Dim, F::Dim>,
+{
+    const NAME: &'static str = "Nelder-Mead";
+
+    type Error = NelderMeadError;
+
+    fn next<Sx, Sfx>(
+        &mut self,
+        f: &F,
+        dom: &Domain<<F>::Scalar>,
+        x: &mut Vector<<F>::Scalar, <F>::Dim, Sx>,
+        fx: &mut Vector<<F>::Scalar, <F>::Dim, Sfx>,
+    ) -> Result<(), Self::Error>
+    where
+        Sx: StorageMut<<F>::Scalar, <F>::Dim>,
+        Sfx: StorageMut<<F>::Scalar, <F>::Dim>,
+    {
+        self.next_inner(f, dom, x).map(|_| {
+            fx.copy_from(&self.fx_best);
+        })
     }
 }
 
